package nckd.jxccl.base.common.utils;

import java.time.DayOfWeek;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.time.Period;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

public class DateUtil {

    // 常用日期格式
    public static final String NORM_DATETIME_PATTERN = "yyyy-MM-dd HH:mm:ss";
    public static final String NORM_DATE_PATTERN = "yyyy-MM-dd";
    public static final String NORM_TIME_PATTERN = "HH:mm:ss";
    private static final DateTimeFormatter NORM_DATETIME_FORMAT = DateTimeFormatter.ofPattern(NORM_DATETIME_PATTERN);
    private static final DateTimeFormatter NORM_DATE_FORMAT = DateTimeFormatter.ofPattern(NORM_DATE_PATTERN);
    private static final DateTimeFormatter NORM_TIME_FORMAT = DateTimeFormatter.ofPattern(NORM_TIME_PATTERN);

    /**
     * 获取当前日期时间
     */
    public static LocalDateTime now() {
        return LocalDateTime.now();
    }

    /**
     * 转换为Date对象
     */
    public static Date toDate(LocalDateTime dateTime) {
        return Date.from(dateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * 转换为LocalDateTime对象
     */
    public static LocalDateTime toLocalDateTime(Date date) {
        return LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());
    }

    /**
     * 格式化日期时间
     */
    public static String format(Date dateTime, String pattern) {
        return format(toLocalDateTime(dateTime),pattern);
    }
    /**
     * 格式化日期时间
     */
    public static String format(LocalDateTime dateTime, String pattern) {
        return dateTime.format(DateTimeFormatter.ofPattern(pattern));
    }

    public static String format(LocalDateTime dateTime) {
        return dateTime.format(NORM_DATETIME_FORMAT);
    }

    /**
     * 解析日期时间
     */
    public static LocalDateTime parse(String dateStr, String pattern) {
        return LocalDateTime.parse(dateStr, DateTimeFormatter.ofPattern(pattern));
    }

    public static LocalDateTime parse(String dateStr) {
        return parse(dateStr, NORM_DATETIME_PATTERN);
    }

    /**
     * 日期加减
     */
    public static LocalDateTime addYears(LocalDateTime dateTime, int years) {
        return dateTime.plusYears(years);
    }

    public static LocalDateTime addMonths(LocalDateTime dateTime, int months) {
        return dateTime.plusMonths(months);
    }

    public static LocalDateTime addDays(LocalDateTime dateTime, int days) {
        return dateTime.plusDays(days);
    }

    public static LocalDateTime addHours(LocalDateTime dateTime, int hours) {
        return dateTime.plusHours(hours);
    }

    public static LocalDateTime addMinutes(LocalDateTime dateTime, int minutes) {
        return dateTime.plusMinutes(minutes);
    }

    public static LocalDateTime addSeconds(LocalDateTime dateTime, int seconds) {
        return dateTime.plusSeconds(seconds);
    }

    /**
     * 获取日期部分
     */

    public static int getYear(Date date) {
        return getYear(toLocalDateTime(date));
    }
    public static int getYear(LocalDateTime dateTime) {
        return dateTime.getYear();
    }

    public static Month getMonth(LocalDateTime dateTime) {
        return dateTime.getMonth();
    }

    public static int getDayOfMonth(LocalDateTime dateTime) {
        return dateTime.getDayOfMonth();
    }

    public static DayOfWeek getDayOfWeek(LocalDateTime dateTime) {
        return dateTime.getDayOfWeek();
    }

    public static int getHour(LocalDateTime dateTime) {
        return dateTime.getHour();
    }

    public static int getMinute(LocalDateTime dateTime) {
        return dateTime.getMinute();
    }

    public static int getSecond(LocalDateTime dateTime) {
        return dateTime.getSecond();
    }

    /**
     * 计算日期差
     */
    public static long between(LocalDateTime start, LocalDateTime end, ChronoUnit unit) {
        return unit.between(start, end);
    }

    /**
     * 判断日期是否在范围内
     */
    public static boolean isInRange(Date date, Date start, Date end){
        return isInRange(toLocalDateTime(date), toLocalDateTime(start), toLocalDateTime(end));
    }
    /**
     * 判断日期是否在范围内
     */
    public static boolean isInRange(LocalDateTime date, LocalDateTime start, LocalDateTime end) {
        return !date.isBefore(start) && !date.isAfter(end);
    }

    /**
     * 获取开始时间
     */
    public static LocalDateTime beginOfDay(LocalDateTime dateTime) {
        return dateTime.toLocalDate().atStartOfDay();
    }

    public static LocalDateTime beginOfMonth(LocalDateTime dateTime) {
        return dateTime.withDayOfMonth(1).toLocalDate().atStartOfDay();
    }

    public static LocalDateTime beginOfYear(LocalDateTime dateTime) {
        return dateTime.withDayOfYear(1).toLocalDate().atStartOfDay();
    }

    /**
     * 获取结束时间
     */
    public static LocalDateTime endOfDay(LocalDateTime dateTime) {
        return dateTime.toLocalDate().atTime(LocalTime.MAX);
    }

    public static LocalDateTime endOfMonth(LocalDateTime dateTime) {
        return dateTime.with(TemporalAdjusters.lastDayOfMonth()).toLocalDate().atTime(LocalTime.MAX);
    }

    public static LocalDateTime endOfYear(LocalDateTime dateTime) {
        return dateTime.with(TemporalAdjusters.lastDayOfYear()).toLocalDate().atTime(LocalTime.MAX);
    }

    /**
     * 生成日期范围
     */
    public static List<LocalDateTime> range(LocalDateTime start, LocalDateTime end, ChronoUnit unit) {
        List<LocalDateTime> result = new ArrayList<>();
        long distance = unit.between(start, end);
        
        for (long i = 0; i <= distance; i++) {
            result.add(start.plus(i, unit));
        }
        return result;
    }

    /**
     * 日期范围处理
     */
    public static <T> List<T> rangeFunc(LocalDateTime start, LocalDateTime end, ChronoUnit unit, Function<LocalDateTime, T> func) {
        return range(start, end, unit).stream().map(func).collect(Collectors.toList());
    }

    public static void rangeConsume(LocalDateTime start, LocalDateTime end, ChronoUnit unit, Consumer<LocalDateTime> consumer) {
        range(start, end, unit).forEach(consumer);
    }

    /**
     * 判断是否闰年
     */
    public static boolean isLeapYear(LocalDateTime dateTime) {
        return dateTime.toLocalDate().isLeapYear();
    }

    /**
     * 时间戳转换
     */
    public static LocalDateTime fromTimestamp(long timestamp) {
        return LocalDateTime.ofInstant(Instant.ofEpochMilli(timestamp), ZoneId.systemDefault());
    }

    public static long toTimestamp(LocalDateTime dateTime) {
        return dateTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    /**
     * 时区转换
     */
    public static LocalDateTime convertTimeZone(LocalDateTime dateTime, ZoneId targetZone) {
        return dateTime.atZone(ZoneId.systemDefault()).withZoneSameInstant(targetZone).toLocalDateTime();
    }

    /**
     * 比较日期
     */
    public static int compare(LocalDateTime dt1, LocalDateTime dt2) {
        return dt1.compareTo(dt2);
    }

    /**
     * 判断同一天
     */
    public static boolean isSameDay(LocalDateTime dt1, LocalDateTime dt2) {
        return dt1.toLocalDate().equals(dt2.toLocalDate());
    }

    /**
     * 年龄计算
     */
    public static int calculateAge(LocalDate birthDate) {
        return Period.between(birthDate, LocalDate.now()).getYears();
    }

    /**
     * 时间段是否重叠
     */
    public static boolean isOverlap(
        LocalDateTime start1, LocalDateTime end1,
        LocalDateTime start2, LocalDateTime end2
    ) {
        return !start1.isAfter(end2) && !start2.isAfter(end1);
    }

    /**
     * 季度处理
     */
    public static int getQuarter(LocalDateTime dateTime) {
        return (dateTime.getMonthValue() - 1) / 3 + 1;
    }

    /**
     * 星座计算
     */
    public static String getZodiac(LocalDateTime dateTime) {
        int month = dateTime.getMonthValue();
        int day = dateTime.getDayOfMonth();
        
        if ((month == 3 && day >= 21) || (month == 4 && day <= 19)) {return "白羊座";};
        if ((month == 4 && day >= 20) || (month == 5 && day <= 20)) {return "金牛座";};
        if ((month == 5 && day >= 21) || (month == 6 && day <= 21)) {return "双子座";};
        if ((month == 6 && day >= 22) || (month == 7 && day <= 22)) {return "巨蟹座";};
        if ((month == 7 && day >= 23) || (month == 8 && day <= 22)) {return "狮子座";};
        if ((month == 8 && day >= 23) || (month == 9 && day <= 22)) {return "处女座";};
        if ((month == 9 && day >= 23) || (month == 10 && day <= 23)) {return "天秤座";};
        if ((month == 10 && day >= 24) || (month == 11 && day <= 22)) {return "天蝎座";};
        if ((month == 11 && day >= 23) || (month == 12 && day <= 21)) {return "射手座";};
        if ((month == 12 && day >= 22) || (month == 1 && day <= 19)) {return "摩羯座";};
        if ((month == 1 && day >= 20) || (month == 2 && day <= 18)) {return "水瓶座";};
        return "双鱼座";
    }

    /**
     * 生肖计算
     */
    public static String getChineseZodiac(LocalDateTime dateTime) {
        String[] zodiacs = {"猴", "鸡", "狗", "猪", "鼠", "牛", "虎", "兔", "龙", "蛇", "马", "羊"};
        return zodiacs[dateTime.getYear() % 12];
    }

    /**
     * 工作日计算
     */
    public static boolean isWorkday(LocalDateTime dateTime) {
        return dateTime.getDayOfWeek().getValue() < 6;
    }

    /**
     * 获取月末最后一天
     */
    public static int lastDayOfMonth(LocalDateTime dateTime) {
        return dateTime.with(TemporalAdjusters.lastDayOfMonth()).getDayOfMonth();
    }

}